package com.muyu.test;

import com.muyu.access.data.base.DataValue;
import com.muyu.basic.BasicEngine;
import com.muyu.domain.EngineVersion;
import org.springframework.stereotype.Service;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author: DongZeLiang
 * @date: 2024/8/29
 * @Description: 测试类
 * @Version: 1.0
 */
@Service
public class TestService {

    static Map<String, BasicEngine<DataValue>> engineMap = new ConcurrentHashMap<>();

    public static final String engineWorkSourcePath = "/home/lib";
    public static final String engineWorkClassPath = engineWorkSourcePath;
    public static final String className = "com.muyu.abstracts.generate.";


    //            package com.muyu.data.engine.service;
    public static final String code =
            """
                    package com.muyu.data.engine.service;
                                
                    import com.muyu.data.engine.basic.abstracts.DataEngineValueActuator;
                    import com.muyu.etl.core.domain.DataValue;
                                
                    /**
                     * @Author: DongZeLiang
                     * @date: 2024/8/29
                     * @Description: 判空
                     * @Version: 1.0
                     */
                    public class ENGINE_VALUE_JDIES979972_V2 extends DataEngineValueActuator {
                                
                        @Override
                        public void run () {
                            DataValue dataValue = get();
                            if (dataValue.getValue() == null){
                                System.out.println("数据为空，需要丢弃");
                            }else{
                                System.out.println("数据非空："+dataValue.getValue());
                            }
                        }
                    }
                    """;


    public static void test(EngineVersion engineVersion) {
        try {
            FileWriter writer;
            writer = new FileWriter(engineWorkSourcePath + engineVersion.getVersionCode());
            writer.write(code);
            writer.flush();
            writer.close();
            // 获取java编译器
            JavaCompiler javaCompiler = ToolProvider.getSystemJavaCompiler();
            InputStream first = null; // 程序的输入  null 用 system.in
            OutputStream second = null; // 程序的输出 null 用 system.out
            OutputStream third = null;  // 程序的错误输出 .,null 用 system.err
            // 程序编译参数 注意 我们编译目录是我们的项目目录
            String[] strings = {"-classpath", engineWorkSourcePath, "-verbose", "-d", engineWorkSourcePath, engineWorkSourcePath + engineVersion.getName()+".java"};
            // 0 表示成功， 其他表示出现了错误
            System.out.println(Arrays.toString(strings));
            int i = javaCompiler.run(first, second, third, strings);
            if (i == 0) {
                System.out.println("成功");
            } else {
                System.out.println("错误");
            }


            // 假设这是你的外部类文件路径
            String externalClassFilePath =
                    engineWorkSourcePath + "\\com\\muyu\\abstracts\\"+engineVersion.getName()+".class";
            Path classFilePath = Paths.get(externalClassFilePath);
            // 获取外部类所在的目录
            String externalClassDir = externalClassFilePath.substring(0, externalClassFilePath.lastIndexOf('\\'));
            URL[] urls = new URL[]{new File(externalClassDir).toURI().toURL()
            };

            // 创建自定义类加载器
            ExternalClassLoader externalClassLoader = new ExternalClassLoader(urls);

            // 加载类
            // 注意类名必须是完全限定名（包括包名）
            Class<?> clazz = externalClassLoader.loadClassFromPath(classFilePath, className);
            // 创建类的实例
            Object instance = clazz.getDeclaredConstructor().newInstance();

            engineMap.put(engineVersion.getName(), (BasicEngine<DataValue>) instance);

        } catch (IllegalAccessException | IOException | InstantiationException |
                 NoSuchMethodException | InvocationTargetException e) {
            throw new RuntimeException(e);
        }
        DataValue dataValue = DataValue.builder()
                .type("String")
                .label("姓名")
                .key("name")
                .value("张三")
                .build();

        BasicEngine<DataValue> engine = engineMap.get(engineVersion.getName());
        engine.set(dataValue);
        engine.execution();
    }
}
